#ifndef _BT_DWARF_H
#define _BT_DWARF_H

#include "asset.h"
#include "error.h"
#include "misc.h"
#include "tree.h"
#include "register.h"

#include <complex.h>
#include <ck_hs.h>
#include <limits.h>
#include <stdbool.h>
#include <stdint.h>
#include <sys/queue.h>

/*
 * Expression operations.
 */
#define BT_DW_OP_addr                      0x03
#define BT_DW_OP_deref                     0x06
#define BT_DW_OP_const1u                   0x08
#define BT_DW_OP_const1s                   0x09
#define BT_DW_OP_const2u                   0x0a
#define BT_DW_OP_const2s                   0x0b
#define BT_DW_OP_const4u                   0x0c
#define BT_DW_OP_const4s                   0x0d
#define BT_DW_OP_const8u                   0x0e
#define BT_DW_OP_const8s                   0x0f
#define BT_DW_OP_constu                    0x10
#define BT_DW_OP_consts                    0x11
#define BT_DW_OP_dup                       0x12
#define BT_DW_OP_drop                      0x13
#define BT_DW_OP_over                      0x14
#define BT_DW_OP_pick                      0x15
#define BT_DW_OP_swap                      0x16
#define BT_DW_OP_rot                       0x17
#define BT_DW_OP_xderef                    0x18
#define BT_DW_OP_abs                       0x19
#define BT_DW_OP_and                       0x1a
#define BT_DW_OP_div                       0x1b
#define BT_DW_OP_minus                     0x1c
#define BT_DW_OP_mod                       0x1d
#define BT_DW_OP_mul                       0x1e
#define BT_DW_OP_neg                       0x1f
#define BT_DW_OP_not                       0x20
#define BT_DW_OP_or                        0x21
#define BT_DW_OP_plus                      0x22
#define BT_DW_OP_plus_uconst               0x23
#define BT_DW_OP_shl                       0x24
#define BT_DW_OP_shr                       0x25
#define BT_DW_OP_shra                      0x26
#define BT_DW_OP_xor                       0x27
#define BT_DW_OP_bra                       0x28
#define BT_DW_OP_eq                        0x29
#define BT_DW_OP_ge                        0x2a
#define BT_DW_OP_gt                        0x2b
#define BT_DW_OP_le                        0x2c
#define BT_DW_OP_lt                        0x2d
#define BT_DW_OP_ne                        0x2e
#define BT_DW_OP_skip                      0x2f
#define BT_DW_OP_lit0                      0x30
#define BT_DW_OP_lit1                      0x31
#define BT_DW_OP_lit2                      0x32
#define BT_DW_OP_lit3                      0x33
#define BT_DW_OP_lit4                      0x34
#define BT_DW_OP_lit5                      0x35
#define BT_DW_OP_lit6                      0x36
#define BT_DW_OP_lit7                      0x37
#define BT_DW_OP_lit8                      0x38
#define BT_DW_OP_lit9                      0x39
#define BT_DW_OP_lit10                     0x3a
#define BT_DW_OP_lit11                     0x3b
#define BT_DW_OP_lit12                     0x3c
#define BT_DW_OP_lit13                     0x3d
#define BT_DW_OP_lit14                     0x3e
#define BT_DW_OP_lit15                     0x3f
#define BT_DW_OP_lit16                     0x40
#define BT_DW_OP_lit17                     0x41
#define BT_DW_OP_lit18                     0x42
#define BT_DW_OP_lit19                     0x43
#define BT_DW_OP_lit20                     0x44
#define BT_DW_OP_lit21                     0x45
#define BT_DW_OP_lit22                     0x46
#define BT_DW_OP_lit23                     0x47
#define BT_DW_OP_lit24                     0x48
#define BT_DW_OP_lit25                     0x49
#define BT_DW_OP_lit26                     0x4a
#define BT_DW_OP_lit27                     0x4b
#define BT_DW_OP_lit28                     0x4c
#define BT_DW_OP_lit29                     0x4d
#define BT_DW_OP_lit30                     0x4e
#define BT_DW_OP_lit31                     0x4f
#define BT_DW_OP_reg0                      0x50
#define BT_DW_OP_reg1                      0x51
#define BT_DW_OP_reg2                      0x52
#define BT_DW_OP_reg3                      0x53
#define BT_DW_OP_reg4                      0x54
#define BT_DW_OP_reg5                      0x55
#define BT_DW_OP_reg6                      0x56
#define BT_DW_OP_reg7                      0x57
#define BT_DW_OP_reg8                      0x58
#define BT_DW_OP_reg9                      0x59
#define BT_DW_OP_reg10                     0x5a
#define BT_DW_OP_reg11                     0x5b
#define BT_DW_OP_reg12                     0x5c
#define BT_DW_OP_reg13                     0x5d
#define BT_DW_OP_reg14                     0x5e
#define BT_DW_OP_reg15                     0x5f
#define BT_DW_OP_reg16                     0x60
#define BT_DW_OP_reg17                     0x61
#define BT_DW_OP_reg18                     0x62
#define BT_DW_OP_reg19                     0x63
#define BT_DW_OP_reg20                     0x64
#define BT_DW_OP_reg21                     0x65
#define BT_DW_OP_reg22                     0x66
#define BT_DW_OP_reg23                     0x67
#define BT_DW_OP_reg24                     0x68
#define BT_DW_OP_reg25                     0x69
#define BT_DW_OP_reg26                     0x6a
#define BT_DW_OP_reg27                     0x6b
#define BT_DW_OP_reg28                     0x6c
#define BT_DW_OP_reg29                     0x6d
#define BT_DW_OP_reg30                     0x6e
#define BT_DW_OP_reg31                     0x6f
#define BT_DW_OP_breg0                     0x70
#define BT_DW_OP_breg1                     0x71
#define BT_DW_OP_breg2                     0x72
#define BT_DW_OP_breg3                     0x73
#define BT_DW_OP_breg4                     0x74
#define BT_DW_OP_breg5                     0x75
#define BT_DW_OP_breg6                     0x76
#define BT_DW_OP_breg7                     0x77
#define BT_DW_OP_breg8                     0x78
#define BT_DW_OP_breg9                     0x79
#define BT_DW_OP_breg10                    0x7a
#define BT_DW_OP_breg11                    0x7b
#define BT_DW_OP_breg12                    0x7c
#define BT_DW_OP_breg13                    0x7d
#define BT_DW_OP_breg14                    0x7e
#define BT_DW_OP_breg15                    0x7f
#define BT_DW_OP_breg16                    0x80
#define BT_DW_OP_breg17                    0x81
#define BT_DW_OP_breg18                    0x82
#define BT_DW_OP_breg19                    0x83
#define BT_DW_OP_breg20                    0x84
#define BT_DW_OP_breg21                    0x85
#define BT_DW_OP_breg22                    0x86
#define BT_DW_OP_breg23                    0x87
#define BT_DW_OP_breg24                    0x88
#define BT_DW_OP_breg25                    0x89
#define BT_DW_OP_breg26                    0x8a
#define BT_DW_OP_breg27                    0x8b
#define BT_DW_OP_breg28                    0x8c
#define BT_DW_OP_breg29                    0x8d
#define BT_DW_OP_breg30                    0x8e
#define BT_DW_OP_breg31                    0x8f
#define BT_DW_OP_regx                      0x90
#define BT_DW_OP_fbreg                     0x91
#define BT_DW_OP_bregx                     0x92
#define BT_DW_OP_piece                     0x93
#define BT_DW_OP_deref_size                0x94
#define BT_DW_OP_xderef_size               0x95
#define BT_DW_OP_nop                       0x96
#define BT_DW_OP_push_object_address       0x97 /* DWARF3 */
#define BT_DW_OP_call2                     0x98 /* DWARF3 */
#define BT_DW_OP_call4                     0x99 /* DWARF3 */
#define BT_DW_OP_call_ref                  0x9a /* DWARF3 */
#define BT_DW_OP_form_tls_address          0x9b /* DWARF3f */
#define BT_DW_OP_call_frame_cfa            0x9c /* DWARF3f */
#define BT_DW_OP_bit_piece                 0x9d /* DWARF3f */
#define BT_DW_OP_implicit_value            0x9e /* DWARF4 */
#define BT_DW_OP_stack_value               0x9f /* DWARF4 */
#define BT_DW_OP_entry_value               0xa1 /* DWARF5 */
#define BT_DW_OP_GNU_push_tls_address      0xe0 /* GNU */
#define BT_DW_OP_lo_user                   0xe0
#define BT_DW_OP_GNU_uninit                0xf0 /* GNU */
#define BT_DW_OP_GNU_encoded_addr          0xf1 /* GNU */
#define BT_DW_OP_GNU_implicit_pointer      0xf2 /* GNU */
#define BT_DW_OP_GNU_entry_value	   0xf3 /* GNU */
#define BT_DW_OP_convert		   0xf7 /* GNU */
#define BT_DW_OP_GNU_parameter_ref	   0xfa
#define BT_DW_OP_HP_unknown                0xe0 /* HP conflict: GNU */
#define BT_DW_OP_HP_is_value               0xe1 /* HP */
#define BT_DW_OP_HP_fltconst4              0xe2 /* HP */
#define BT_DW_OP_HP_fltconst8              0xe3 /* HP */
#define BT_DW_OP_HP_mod_range              0xe4 /* HP */
#define BT_DW_OP_HP_unmod_range            0xe5 /* HP */
#define BT_DW_OP_HP_tls                    0xe6 /* HP */
#define BT_DW_OP_INTEL_bit_piece           0xe8
#define BT_DW_OP_APPLE_uninit              0xf0 /* Apple */
#define BT_DW_OP_PGI_omp_thread_num        0xf8 /* PGI (STMicroelectronics) */
#define BT_DW_OP_hi_user                   0xff

/*
 * Array ordering.
 */
#define DW_ORD_row_major                0x00
#define DW_ORD_col_major                0x01

/*
 * Attribute types.
 */
#define BT_DW_AT_sibling                           0x01
#define BT_DW_AT_location                          0x02
#define BT_DW_AT_name                              0x03
#define BT_DW_AT_ordering                          0x09
#define BT_DW_AT_subscr_data                       0x0a
#define BT_DW_AT_byte_size                         0x0b
#define BT_DW_AT_bit_offset                        0x0c
#define BT_DW_AT_bit_size                          0x0d
#define BT_DW_AT_element_list                      0x0f
#define BT_DW_AT_stmt_list                         0x10
#define BT_DW_AT_low_pc                            0x11
#define BT_DW_AT_high_pc                           0x12
#define BT_DW_AT_language                          0x13
#define BT_DW_AT_member                            0x14
#define BT_DW_AT_discr                             0x15
#define BT_DW_AT_discr_value                       0x16
#define BT_DW_AT_visibility                        0x17
#define BT_DW_AT_import                            0x18
#define BT_DW_AT_string_length                     0x19
#define BT_DW_AT_common_reference                  0x1a
#define BT_DW_AT_comp_dir                          0x1b
#define BT_DW_AT_const_value                       0x1c
#define BT_DW_AT_containing_type                   0x1d
#define BT_DW_AT_default_value                     0x1e
#define BT_DW_AT_inline                            0x20
#define BT_DW_AT_is_optional                       0x21
#define BT_DW_AT_lower_bound                       0x22
#define BT_DW_AT_producer                          0x25
#define BT_DW_AT_prototyped                        0x27
#define BT_DW_AT_return_addr                       0x2a
#define BT_DW_AT_start_scope                       0x2c
#define BT_DW_AT_bit_stride                        0x2e /* DWARF3 name */
#define BT_DW_AT_stride_size                       0x2e /* DWARF2 name */
#define BT_DW_AT_upper_bound                       0x2f
#define BT_DW_AT_abstract_origin                   0x31
#define BT_DW_AT_accessibility                     0x32
#define BT_DW_AT_address_class                     0x33
#define BT_DW_AT_artificial                        0x34
#define BT_DW_AT_base_types                        0x35
#define BT_DW_AT_calling_convention                0x36
#define BT_DW_AT_count                             0x37
#define BT_DW_AT_data_member_location              0x38
#define BT_DW_AT_decl_column                       0x39
#define BT_DW_AT_decl_file                         0x3a
#define BT_DW_AT_decl_line                         0x3b
#define BT_DW_AT_declaration                       0x3c
#define BT_DW_AT_discr_list                        0x3d
#define BT_DW_AT_encoding                          0x3e
#define BT_DW_AT_external                          0x3f
#define BT_DW_AT_frame_base                        0x40
#define BT_DW_AT_friend                            0x41
#define BT_DW_AT_identifier_case                   0x42
#define BT_DW_AT_macro_info                        0x43
#define BT_DW_AT_namelist_item                     0x44
#define BT_DW_AT_priority                          0x45
#define BT_DW_AT_segment                           0x46
#define BT_DW_AT_specification                     0x47
#define BT_DW_AT_static_link                       0x48
#define BT_DW_AT_type                              0x49
#define BT_DW_AT_use_location                      0x4a
#define BT_DW_AT_variable_parameter                0x4b
#define BT_DW_AT_virtuality                        0x4c
#define BT_DW_AT_vtable_elem_location              0x4d
#define BT_DW_AT_allocated                         0x4e /* DWARF3 */
#define BT_DW_AT_associated                        0x4f /* DWARF3 */
#define BT_DW_AT_data_location                     0x50 /* DWARF3 */
#define BT_DW_AT_byte_stride                       0x51 /* DWARF3f */
#define BT_DW_AT_stride                            0x51 /* DWARF3 (do not use) */
#define BT_DW_AT_entry_pc                          0x52 /* DWARF3 */
#define BT_DW_AT_use_UTF8                          0x53 /* DWARF3 */
#define BT_DW_AT_extension                         0x54 /* DWARF3 */
#define BT_DW_AT_ranges                            0x55 /* DWARF3 */
#define BT_DW_AT_trampoline                        0x56 /* DWARF3 */
#define BT_DW_AT_call_column                       0x57 /* DWARF3 */
#define BT_DW_AT_call_file                         0x58 /* DWARF3 */
#define BT_DW_AT_call_line                         0x59 /* DWARF3 */
#define BT_DW_AT_description                       0x5a /* DWARF3 */
#define BT_DW_AT_binary_scale                      0x5b /* DWARF3f */
#define BT_DW_AT_decimal_scale                     0x5c /* DWARF3f */
#define BT_DW_AT_small                             0x5d /* DWARF3f */
#define BT_DW_AT_decimal_sign                      0x5e /* DWARF3f */
#define BT_DW_AT_digit_count                       0x5f /* DWARF3f */
#define BT_DW_AT_picture_string                    0x60 /* DWARF3f */
#define BT_DW_AT_mutable                           0x61 /* DWARF3f */
#define BT_DW_AT_threads_scaled                    0x62 /* DWARF3f */
#define BT_DW_AT_explicit                          0x63 /* DWARF3f */
#define BT_DW_AT_object_pointer                    0x64 /* DWARF3f */
#define BT_DW_AT_endianity                         0x65 /* DWARF3f */
#define BT_DW_AT_elemental                         0x66 /* DWARF3f */
#define BT_DW_AT_pure                              0x67 /* DWARF3f */
#define BT_DW_AT_recursive                         0x68 /* DWARF3f */
#define BT_DW_AT_signature                         0x69 /* DWARF4 */
#define BT_DW_AT_main_subprogram                   0x6a /* DWARF4 */
#define BT_DW_AT_data_bit_offset                   0x6b /* DWARF4 */
#define BT_DW_AT_const_expr                        0x6c /* DWARF4 */
#define BT_DW_AT_enum_class                        0x6d /* DWARF4 */
#define BT_DW_AT_linkage_name                      0x6e /* DWARF4 */
#define BT_DW_AT_call_site_value                   0x6f /* DWARF5 */
#define BT_DW_AT_call_site_data_value              0x70 /* DWARF5 */
#define BT_DW_AT_call_site_data_location           0x71 /* DWARF5 */
#define BT_DW_AT_call_site_target                  0x72 /* DWARF5 */
#define BT_DW_AT_call_site_target_clobbered        0x73 /* DWARF5 */
#define BT_DW_AT_call_site_pc                      0x74 /* DWARF5 */
#define BT_DW_AT_call_site_return_pc               0x75 /* DWARF5 */
#define BT_DW_AT_call_site_origin                  0x76 /* DWARF5 */
#define BT_DW_AT_call_site_parameter               0x77 /* DWARF5 */
#define BT_DW_AT_tail_call                         0x78 /* DWARF5 */
#define BT_DW_AT_all_tail_call_sites               0x79 /* DWARF5 */
#define BT_DW_AT_all_call_sites                    0x7a /* DWARF5 */
#define BT_DW_AT_all_source_call_sites             0x7b /* DWARF5 */
#define BT_DW_AT_HP_block_index                    0x2000  /* HP */
#define BT_DW_AT_lo_user                           0x2000
#define BT_DW_AT_MIPS_fde                          0x2001 /* MIPS/SGI */
#define BT_DW_AT_MIPS_loop_begin                   0x2002 /* MIPS/SGI */
#define BT_DW_AT_MIPS_tail_loop_begin              0x2003 /* MIPS/SGI */
#define BT_DW_AT_MIPS_epilog_begin                 0x2004 /* MIPS/SGI */
#define BT_DW_AT_MIPS_loop_unroll_factor           0x2005 /* MIPS/SGI */
#define BT_DW_AT_MIPS_software_pipeline_depth      0x2006 /* MIPS/SGI */
#define BT_DW_AT_MIPS_linkage_name                 0x2007 /* MIPS/SGI, GNU, and others.*/
#define BT_DW_AT_MIPS_stride                       0x2008 /* MIPS/SGI */
#define BT_DW_AT_MIPS_abstract_name                0x2009 /* MIPS/SGI */
#define BT_DW_AT_MIPS_clone_origin                 0x200a /* MIPS/SGI */
#define BT_DW_AT_MIPS_has_inlines                  0x200b /* MIPS/SGI */
#define BT_DW_AT_MIPS_stride_byte                  0x200c /* MIPS/SGI */
#define BT_DW_AT_MIPS_stride_elem                  0x200d /* MIPS/SGI */
#define BT_DW_AT_MIPS_ptr_dopetype                 0x200e /* MIPS/SGI */
#define BT_DW_AT_MIPS_allocatable_dopetype         0x200f /* MIPS/SGI */
#define BT_DW_AT_MIPS_assumed_shape_dopetype       0x2010 /* MIPS/SGI */
#define BT_DW_AT_MIPS_assumed_size                 0x2011 /* MIPS/SGI */
#define BT_DW_AT_HP_unmodifiable                   0x2001 /* conflict: MIPS */
#define BT_DW_AT_HP_actuals_stmt_list              0x2010 /* conflict: MIPS */
#define BT_DW_AT_HP_proc_per_section               0x2011 /* conflict: MIPS */
#define BT_DW_AT_HP_raw_data_ptr                   0x2012 /* HP */
#define BT_DW_AT_HP_pass_by_reference              0x2013 /* HP */
#define BT_DW_AT_HP_opt_level                      0x2014 /* HP */
#define BT_DW_AT_HP_prof_version_id                0x2015 /* HP */
#define BT_DW_AT_HP_opt_flags                      0x2016 /* HP */
#define BT_DW_AT_HP_cold_region_low_pc             0x2017 /* HP */
#define BT_DW_AT_HP_cold_region_high_pc            0x2018 /* HP */
#define BT_DW_AT_HP_all_variables_modifiable       0x2019 /* HP */
#define BT_DW_AT_HP_linkage_name                   0x201a /* HP */
#define BT_DW_AT_HP_prof_flags                     0x201b /* HP */
#define BT_DW_AT_CPQ_discontig_ranges              0x2001 /* COMPAQ/HP */
#define BT_DW_AT_CPQ_semantic_events               0x2002 /* COMPAQ/HP */
#define BT_DW_AT_CPQ_split_lifetimes_var           0x2003 /* COMPAQ/HP */
#define BT_DW_AT_CPQ_split_lifetimes_rtn           0x2004 /* COMPAQ/HP */
#define BT_DW_AT_CPQ_prologue_length               0x2005 /* COMPAQ/HP */
#define BT_DW_AT_INTEL_other_endian                0x2026 /* Intel, 1 if byte swapped. */
#define BT_DW_AT_sf_names                          0x2101 /* GNU */
#define BT_DW_AT_src_info                          0x2102 /* GNU */
#define BT_DW_AT_mac_info                          0x2103 /* GNU */
#define BT_DW_AT_src_coords                        0x2104 /* GNU */
#define BT_DW_AT_body_begin                        0x2105 /* GNU */
#define BT_DW_AT_body_end                          0x2106 /* GNU */
#define BT_DW_AT_GNU_vector                        0x2107 /* GNU */
#define BT_DW_AT_GNU_guarded_by                    0x2108 /* GNU */
#define BT_DW_AT_GNU_pt_guarded_by                 0x2109 /* GNU */
#define BT_DW_AT_GNU_guarded                       0x210a /* GNU */
#define BT_DW_AT_GNU_pt_guarded                    0x210b /* GNU */
#define BT_DW_AT_GNU_locks_excluded                0x210c /* GNU */
#define BT_DW_AT_GNU_exclusive_locks_required      0x210d /* GNU */
#define BT_DW_AT_GNU_shared_locks_required         0x210e /* GNU */
#define BT_DW_AT_GNU_odr_signature                 0x210f /* GNU */
#define BT_DW_AT_GNU_template_name                 0x2110 /* GNU */
#define BT_DW_AT_GNU_call_site_value               0x2111 /* GNU */
#define BT_DW_AT_GNU_call_site_data_value          0x2112 /* GNU */
#define BT_DW_AT_GNU_call_site_target              0x2113 /* GNU */
#define BT_DW_AT_GNU_call_site_target_clobbered    0x2114 /* GNU */
#define BT_DW_AT_GNU_tail_call                     0x2115 /* GNU */
#define BT_DW_AT_GNU_all_tail_call_sites           0x2116 /* GNU */
#define BT_DW_AT_GNU_all_call_sites                0x2117 /* GNU */
#define BT_DW_AT_GNU_all_source_call_sites         0x2118 /* GNU */
#define BT_DW_AT_ALTIUM_loclist                    0x2300 /* ALTIUM  */
#define BT_DW_AT_SUN_template                      0x2201 /* SUN */
#define BT_DW_AT_VMS_rtnbeg_pd_address             0x2201 /* VMS */
#define BT_DW_AT_SUN_alignment                     0x2202 /* SUN */
#define BT_DW_AT_SUN_vtable                        0x2203 /* SUN */
#define BT_DW_AT_SUN_count_guarantee               0x2204 /* SUN */
#define BT_DW_AT_SUN_command_line                  0x2205 /* SUN */
#define BT_DW_AT_SUN_vbase                         0x2206 /* SUN */
#define BT_DW_AT_SUN_compile_options               0x2207 /* SUN */
#define BT_DW_AT_SUN_language                      0x2208 /* SUN */
#define BT_DW_AT_SUN_browser_file                  0x2209 /* SUN */
#define BT_DW_AT_SUN_vtable_abi                    0x2210 /* SUN */
#define BT_DW_AT_SUN_func_offsets                  0x2211 /* SUN */
#define BT_DW_AT_SUN_cf_kind                       0x2212 /* SUN */
#define BT_DW_AT_SUN_vtable_index                  0x2213 /* SUN */
#define BT_DW_AT_SUN_omp_tpriv_addr                0x2214 /* SUN */
#define BT_DW_AT_SUN_omp_child_func                0x2215 /* SUN */
#define BT_DW_AT_SUN_func_offset                   0x2216 /* SUN */
#define BT_DW_AT_SUN_memop_type_ref                0x2217 /* SUN */
#define BT_DW_AT_SUN_profile_id                    0x2218 /* SUN */
#define BT_DW_AT_SUN_memop_signature               0x2219 /* SUN */
#define BT_DW_AT_SUN_obj_dir                       0x2220 /* SUN */
#define BT_DW_AT_SUN_obj_file                      0x2221 /* SUN */
#define BT_DW_AT_SUN_original_name                 0x2222 /* SUN */
#define BT_DW_AT_SUN_hwcprof_signature             0x2223 /* SUN */
#define BT_DW_AT_SUN_amd64_parmdump                0x2224 /* SUN */
#define BT_DW_AT_SUN_part_link_name                0x2225 /* SUN */
#define BT_DW_AT_SUN_link_name                     0x2226 /* SUN */
#define BT_DW_AT_SUN_pass_with_const               0x2227 /* SUN */
#define BT_DW_AT_SUN_return_with_const             0x2228 /* SUN */
#define BT_DW_AT_SUN_import_by_name                0x2229 /* SUN */
#define BT_DW_AT_SUN_f90_pointer                   0x222a /* SUN */
#define BT_DW_AT_SUN_pass_by_ref                   0x222b /* SUN */
#define BT_DW_AT_SUN_f90_allocatable               0x222c /* SUN */
#define BT_DW_AT_SUN_f90_assumed_shape_array       0x222d /* SUN */
#define BT_DW_AT_SUN_c_vla                         0x222e /* SUN */
#define BT_DW_AT_SUN_return_value_ptr              0x2230 /* SUN */
#define BT_DW_AT_SUN_dtor_start                    0x2231 /* SUN */
#define BT_DW_AT_SUN_dtor_length                   0x2232 /* SUN */
#define BT_DW_AT_SUN_dtor_state_initial            0x2233 /* SUN */
#define BT_DW_AT_SUN_dtor_state_final              0x2234 /* SUN */
#define BT_DW_AT_SUN_dtor_state_deltas             0x2235 /* SUN */
#define BT_DW_AT_SUN_import_by_lname               0x2236 /* SUN */
#define BT_DW_AT_SUN_f90_use_only                  0x2237 /* SUN */
#define BT_DW_AT_SUN_namelist_spec                 0x2238 /* SUN */
#define BT_DW_AT_SUN_is_omp_child_func             0x2239 /* SUN */
#define BT_DW_AT_SUN_fortran_main_alias            0x223a /* SUN */
#define BT_DW_AT_SUN_fortran_based                 0x223b /* SUN */
/*   See http://gcc.gnu.org/wiki/BT_DW_AT_GNAT_descriptive_type .  */
#define BT_DW_AT_use_GNAT_descriptive_type         0x2301 /* GNAT */
#define BT_DW_AT_GNAT_descriptive_type             0x2302 /* GNAT */
#define BT_DW_AT_upc_threads_scaled                0x3210 /* UPC */
#define BT_DW_AT_PGI_lbase                         0x3a00
#define BT_DW_AT_PGI_soffset                       0x3a01
#define BT_DW_AT_PGI_lstride                       0x3a02
#define BT_DW_AT_APPLE_optimized                   0x3fe1 /* Apple */
#define BT_DW_AT_APPLE_flags                       0x3fe2 /* Apple */
#define BT_DW_AT_APPLE_isa                         0x3fe3 /* Apple */
#define BT_DW_AT_APPLE_block                       0x3fe4 /* Apple */
#define BT_DW_AT_APPLE_major_runtime_vers          0x3fe5 /* Apple */
#define BT_DW_AT_APPLE_runtime_class               0x3fe6 /* Apple */
#define BT_DW_AT_APPLE_omit_frame_ptr              0x3fe7 /* Apple */
#define BT_DW_AT_APPLE_closure                     0x3fe4 /* Apple */
#define BT_DW_AT_APPLE_major_runtime_vers          0x3fe5 /* Apple */
#define BT_DW_AT_APPLE_runtime_class               0x3fe6 /* Apple */
#define BT_DW_AT_hi_user                           0x3fff

#define BT_DW_ATE_address                  0x1
#define BT_DW_ATE_boolean                  0x2
#define BT_DW_ATE_complex_float            0x3
#define BT_DW_ATE_float                    0x4
#define BT_DW_ATE_signed                   0x5
#define BT_DW_ATE_signed_char              0x6
#define BT_DW_ATE_unsigned                 0x7
#define BT_DW_ATE_unsigned_char            0x8
#define BT_DW_ATE_imaginary_float          0x9  /* DWARF3 */
#define BT_DW_ATE_packed_decimal           0xa  /* DWARF3f */
#define BT_DW_ATE_numeric_string           0xb  /* DWARF3f */
#define BT_DW_ATE_edited                   0xc  /* DWARF3f */
#define BT_DW_ATE_signed_fixed             0xd  /* DWARF3f */
#define BT_DW_ATE_unsigned_fixed           0xe  /* DWARF3f */
#define BT_DW_ATE_decimal_float            0xf  /* DWARF3f */
#define BT_DW_ATE_UTF			   0x10 /* DWARF4 */
#define BT_DW_ATE_ALTIUM_fract             0x80 /* ALTIUM __fract type */
#define BT_DW_ATE_lo_user                  0x80
#define BT_DW_ATE_ALTIUM_accum             0x81 /* ALTIUM __accum type */
#define BT_DW_ATE_HP_float80               0x80 /* (80 bit). HP */
#define BT_DW_ATE_HP_complex_float80       0x81 /* Complex (80 bit). HP  */
#define BT_DW_ATE_HP_float128              0x82 /* (128 bit). HP */
#define BT_DW_ATE_HP_complex_float128      0x83 /* Complex (128 bit). HP */
#define BT_DW_ATE_HP_floathpintel          0x84 /* (82 bit IA64). HP */
#define BT_DW_ATE_HP_imaginary_float80     0x85 /* HP */
#define BT_DW_ATE_HP_imaginary_float128    0x86 /* HP */
#define BT_DW_ATE_SUN_interval_float       0x91
#define BT_DW_ATE_SUN_imaginary_float      0x92 /* Obsolete: See BT_DW_ATE_imaginary_float */
#define BT_DW_ATE_hi_user                  0xff
#define BT_DW_ATCF_lo_user                 0x40 /* SUN */
#define BT_DW_ATCF_SUN_mop_bitfield        0x41 /* SUN */
#define BT_DW_ATCF_SUN_mop_spill           0x42 /* SUN */
#define BT_DW_ATCF_SUN_mop_scopy           0x43 /* SUN */
#define BT_DW_ATCF_SUN_func_start          0x44 /* SUN */
#define BT_DW_ATCF_SUN_end_ctors           0x45 /* SUN */
#define BT_DW_ATCF_SUN_branch_target       0x46 /* SUN */
#define BT_DW_ATCF_SUN_mop_stack_probe     0x47 /* SUN */
#define BT_DW_ATCF_SUN_func_epilog         0x48 /* SUN */
#define BT_DW_ATCF_hi_user                 0xff /* SUN */

/*
 * DIE TAG types.
 */
#define BT_DW_TAG_array_type               0x01
#define BT_DW_TAG_class_type               0x02
#define BT_DW_TAG_entry_point              0x03
#define BT_DW_TAG_enumeration_type         0x04
#define BT_DW_TAG_formal_parameter         0x05
#define BT_DW_TAG_imported_declaration     0x08
#define BT_DW_TAG_label                    0x0a
#define BT_DW_TAG_lexical_block            0x0b
#define BT_DW_TAG_member                   0x0d
#define BT_DW_TAG_pointer_type             0x0f
#define BT_DW_TAG_reference_type           0x10
#define BT_DW_TAG_compile_unit             0x11
#define BT_DW_TAG_string_type              0x12
#define BT_DW_TAG_structure_type           0x13
#define BT_DW_TAG_subroutine_type          0x15
#define BT_DW_TAG_typedef                  0x16
#define BT_DW_TAG_union_type               0x17
#define BT_DW_TAG_unspecified_parameters   0x18
#define BT_DW_TAG_variant                  0x19
#define BT_DW_TAG_common_block             0x1a
#define BT_DW_TAG_common_inclusion         0x1b
#define BT_DW_TAG_inheritance              0x1c
#define BT_DW_TAG_inlined_subroutine       0x1d
#define BT_DW_TAG_module                   0x1e
#define BT_DW_TAG_ptr_to_member_type       0x1f
#define BT_DW_TAG_set_type                 0x20
#define BT_DW_TAG_subrange_type            0x21
#define BT_DW_TAG_with_stmt                0x22
#define BT_DW_TAG_access_declaration       0x23
#define BT_DW_TAG_base_type                0x24
#define BT_DW_TAG_catch_block              0x25
#define BT_DW_TAG_const_type               0x26
#define BT_DW_TAG_constant                 0x27
#define BT_DW_TAG_enumerator               0x28
#define BT_DW_TAG_file_type                0x29
#define BT_DW_TAG_friend                   0x2a
#define BT_DW_TAG_namelist                 0x2b
#define BT_DW_TAG_namelist_item            0x2c /* DWARF3/2 spelling */
#define BT_DW_TAG_packed_type              0x2d
#define BT_DW_TAG_subprogram               0x2e
#define BT_DW_TAG_template_type_parameter  0x2f /* DWARF3/2 spelling*/
#define BT_DW_TAG_template_value_parameter 0x30 /* DWARF3/2 spelling*/
#define BT_DW_TAG_thrown_type              0x31
#define BT_DW_TAG_try_block                0x32
#define BT_DW_TAG_variant_part             0x33
#define BT_DW_TAG_variable                 0x34
#define BT_DW_TAG_volatile_type            0x35
#define BT_DW_TAG_dwarf_procedure          0x36  /* DWARF3 */
#define BT_DW_TAG_restrict_type            0x37  /* DWARF3 */
#define BT_DW_TAG_interface_type           0x38  /* DWARF3 */
#define BT_DW_TAG_namespace                0x39  /* DWARF3 */
#define BT_DW_TAG_imported_module          0x3a  /* DWARF3 */
#define BT_DW_TAG_unspecified_type         0x3b  /* DWARF3 */
#define BT_DW_TAG_partial_unit             0x3c  /* DWARF3 */
#define BT_DW_TAG_imported_unit            0x3d  /* DWARF3 */
#define BT_DW_TAG_mutable_type		   0x3e  /* Withdrawn */
#define BT_DW_TAG_condition                0x3f  /* DWARF3f */
#define BT_DW_TAG_shared_type              0x40  /* DWARF3f */
#define BT_DW_TAG_type_unit                0x41  /* DWARF4 */
#define BT_DW_TAG_rvalue_reference_type    0x42  /* DWARF4 */
#define BT_DW_TAG_template_alias           0x43  /* DWARF4 */
#define BT_DW_TAG_call_site                0x44  /* DWARF5 */
#define BT_DW_TAG_call_site_parameter      0x45  /* DWARF5 */
#define BT_DW_TAG_lo_user                  0x4080
#define BT_DW_TAG_MIPS_loop                0x4081
#define BT_DW_TAG_HP_array_descriptor      0x4090 /* HP */
#define BT_DW_TAG_format_label             0x4101 /* GNU. Fortran. */
#define BT_DW_TAG_function_template        0x4102 /* GNU. For C++ */
#define BT_DW_TAG_class_template           0x4103 /* GNU. For C++ */
#define BT_DW_TAG_GNU_BINCL                0x4104 /* GNU */
#define BT_DW_TAG_GNU_EINCL                0x4105 /* GNU */
#define BT_DW_TAG_GNU_template_template_parameter  0x4106 /* GNU */
#define BT_DW_TAG_GNU_template_template_param      0x4106 /* GNU */
#define BT_DW_TAG_GNU_template_parameter_pack      0x4107 /* GNU */
#define BT_DW_TAG_GNU_formal_parameter_pack        0x4108 /* GNU */
#define BT_DW_TAG_GNU_call_site                    0x4109 /* GNU */
#define BT_DW_TAG_GNU_call_site_parameter          0x410a /* GNU */
#define BT_DW_TAG_ALTIUM_circ_type         0x5101 /* ALTIUM */
#define BT_DW_TAG_ALTIUM_mwa_circ_type     0x5102 /* ALTIUM */
#define BT_DW_TAG_ALTIUM_rev_carry_type    0x5103 /* ALTIUM */
#define BT_DW_TAG_ALTIUM_rom               0x5111 /* ALTIUM */
#define BT_DW_TAG_upc_shared_type          0x8765 /* UPC */
#define BT_DW_TAG_upc_strict_type          0x8766 /* UPC */
#define BT_DW_TAG_upc_relaxed_type         0x8767 /* UPC */
#define BT_DW_TAG_PGI_kanji_type           0xa000 /* PGI */
#define BT_DW_TAG_PGI_interface_block      0xa020 /* PGI */
#define BT_DW_TAG_SUN_function_template    0x4201 /* SUN */
#define BT_DW_TAG_SUN_class_template       0x4202 /* SUN */
#define BT_DW_TAG_SUN_struct_template      0x4203 /* SUN */
#define BT_DW_TAG_SUN_union_template       0x4204 /* SUN */
#define BT_DW_TAG_SUN_indirect_inheritance 0x4205 /* SUN */
#define BT_DW_TAG_SUN_codeflags            0x4206 /* SUN */
#define BT_DW_TAG_SUN_memop_info           0x4207 /* SUN */
#define BT_DW_TAG_SUN_omp_child_func       0x4208 /* SUN */
#define BT_DW_TAG_SUN_rtti_descriptor      0x4209 /* SUN */
#define BT_DW_TAG_SUN_dtor_info            0x420a /* SUN */
#define BT_DW_TAG_SUN_dtor                 0x420b /* SUN */
#define BT_DW_TAG_SUN_f90_interface        0x420c /* SUN */
#define BT_DW_TAG_SUN_fortran_vax_structure 0x420d /* SUN */
#define BT_DW_TAG_SUN_hi                   0x42ff /* SUN */
#define BT_DW_TAG_hi_user                  0xffff

/*
 * Attribute forms as of DWARF4 June 10, 2010 specification.
 */
#define BT_DW_FORM_addr                    0x01
#define BT_DW_FORM_block2                  0x03
#define BT_DW_FORM_block4                  0x04
#define BT_DW_FORM_data2                   0x05
#define BT_DW_FORM_data4                   0x06
#define BT_DW_FORM_data8                   0x07
#define BT_DW_FORM_string                  0x08
#define BT_DW_FORM_block                   0x09
#define BT_DW_FORM_block1                  0x0a
#define BT_DW_FORM_data1                   0x0b
#define BT_DW_FORM_flag                    0x0c
#define BT_DW_FORM_sdata                   0x0d
#define BT_DW_FORM_strp                    0x0e
#define BT_DW_FORM_udata                   0x0f
#define BT_DW_FORM_ref_addr                0x10
#define BT_DW_FORM_ref1                    0x11
#define BT_DW_FORM_ref2                    0x12
#define BT_DW_FORM_ref4                    0x13
#define BT_DW_FORM_ref8                    0x14
#define BT_DW_FORM_ref_udata               0x15
#define BT_DW_FORM_indirect                0x16
#define BT_DW_FORM_sec_offset              0x17 /* DWARF4 */
#define BT_DW_FORM_exprloc                 0x18 /* DWARF4 */
#define BT_DW_FORM_flag_present            0x19 /* DWARF4 */
#define BT_DW_FORM_strx			   0x1a	/* DWARF5 */
#define BT_DW_FORM_addrx		   0x1b	/* DWARF5 */
#define BT_DW_FORM_ref_sig8                0x20 /* DWARF4 */
#define BT_DW_FORM_GNU_addr_index	   0x1f01
#define BT_DW_FORM_GNU_str_index	   0x1f02
#define BT_DW_FORM_GNU_ref_alt		   0x1f20
#define BT_DW_FORM_GNU_strp_alt		   0x1f21

/*
 * Attribute form classes
 */
#define BT_DW_FORM_CLS_addr		(1UL << 0)
#define BT_DW_FORM_CLS_block		(1UL << 1)
#define BT_DW_FORM_CLS_constant		(1UL << 2)
#define BT_DW_FORM_CLS_constant_us	(1UL << 3)
#define BT_DW_FORM_CLS_flag		(1UL << 4)
#define BT_DW_FORM_CLS_ref		(1UL << 5)
#define BT_DW_FORM_CLS_sec_offset	(1UL << 6)
#define BT_DW_FORM_CLS_string		(1UL << 7)
#define BT_DW_FORM_CLS_exprloc		(1UL << 8)
#define BT_DW_FORM_CLS_ref_cu		(1UL << 9)

#define BT_DW_CHILDREN_no	0x00
#define BT_DW_CHILDREN_yes	0x01

/*
 * These values pertain to DWARF4 line number programs as of the
 * June 10, 2010 specification.
 */
#define BT_DW_LNS_BEGIN				0x01
#define BT_DW_LNS_copy				0x01
#define BT_DW_LNS_advance_pc			0x02
#define BT_DW_LNS_advance_line			0x03
#define BT_DW_LNS_set_file			0x04
#define BT_DW_LNS_set_column			0x05
#define BT_DW_LNS_negate_stmt			0x06
#define BT_DW_LNS_set_basic_block		0x07
#define BT_DW_LNS_const_add_pc			0x08
#define BT_DW_LNS_fixed_advance_pc		0x09
#define BT_DW_LNS_set_prologue_end		0x0a
#define BT_DW_LNS_set_epilogue_begin		0x0b
#define BT_DW_LNS_set_isa			0x0c
#define BT_DW_LNS_END				0x0c

#define BT_DW_LNE_BEGIN				0x01
#define BT_DW_LNE_end_sequence			0x01
#define BT_DW_LNE_set_address			0x02
#define BT_DW_LNE_define_file			0x03
#define BT_DW_LNE_set_discriminator		0x04
#define BT_DW_LNE_END				0x04
#define BT_DW_LNE_HP_negate_is_UV_update	0x11
#define BT_DW_LNE_HP_push_context		0x12
#define BT_DW_LNE_HP_pop_context		0x13
#define BT_DW_LNE_HP_set_file_line_column	0x14
#define BT_DW_LNE_HP_set_routine_name		0x15
#define BT_DW_LNE_HP_set_sequence		0x16
#define BT_DW_LNE_HP_negate_post_semantics	0x17
#define BT_DW_LNE_HP_negate_function_exit	0x18
#define BT_DW_LNE_HP_negate_front_end_logical	0x19
#define BT_DW_LNE_HP_define_proc		0x20
#define BT_DW_LNE_HP_source_file_correlation	0x80
#define BT_DW_LNE_lo_user			0x80
#define BT_DW_LNE_hi_user			0xff

/*
 * BT_DW_SET_isa values.
 */
#define BT_DW_ISA_UNKNOWN			0x00
#define BT_DW_ISA_ARM_thumb			0x01
#define BT_DW_ISA_ARM_arm			0x02

/*
 * "none" is not actually a value from the DWARF
 * specification, but we use it to indicate that
 * a DIE does not have an accessibility modifier.
 */
#define BT_DW_ACCESS_none			0x00
#define BT_DW_ACCESS_public			0x01
#define BT_DW_ACCESS_protected			0x02
#define BT_DW_ACCESS_private			0x03

#define BT_DW_LANG_C89				0x0001
#define BT_DW_LANG_C				0x0002
#define BT_DW_LANG_Ada83			0x0003
#define BT_DW_LANG_C_plus_plus			0x0004
#define BT_DW_LANG_Cobol74			0x0005
#define BT_DW_LANG_Cobol85			0x0006
#define BT_DW_LANG_Fortran77			0x0007
#define BT_DW_LANG_Fortran90			0x0008
#define BT_DW_LANG_Pascal83			0x0009
#define BT_DW_LANG_Modula2			0x000a
#define BT_DW_LANG_Java				0x000b
#define BT_DW_LANG_C99				0x000c
#define BT_DW_LANG_Ada95			0x000d
#define BT_DW_LANG_Fortran95			0x000e
#define BT_DW_LANG_PLI				0x000f
#define BT_DW_LANG_ObjC				0x0010
#define BT_DW_LANG_ObjC_plus_plus		0x0011
#define BT_DW_LANG_UPC				0x0012
#define BT_DW_LANG_D				0x0013
#define BT_DW_LANG_Go				0x0016

#define BT_DW_LANG_lo_user			0x8000
#define BT_DW_LANG_hi_user			0xffff

#define BT_DW_LANG_Mips_Assembler		0x8001

#define BT_DW_CIE_ID_32				UINT32_MAX
#define BT_DW_CIE_ID_64				UINT64_MAX

/* DWARF CFI opcodes */
#define BT_DW_CFA_advance_loc			0x01
#define BT_DW_CFA_offset			0x02
#define BT_DW_CFA_restore			0x03

/* DWARF CFI initial operands */
#define BT_DW_CFA_nop				0x00
#define BT_DW_CFA_set_loc 			0x01
#define BT_DW_CFA_advance_loc1			0x02
#define BT_DW_CFA_advance_loc2			0x03
#define BT_DW_CFA_advance_loc4			0x04
#define BT_DW_CFA_offset_extended		0x05
#define BT_DW_CFA_restore_extended		0x06
#define BT_DW_CFA_undefined			0x07
#define BT_DW_CFA_same_value			0x08
#define BT_DW_CFA_register			0x09
#define BT_DW_CFA_remember_state		0x0a
#define BT_DW_CFA_restore_state			0x0b
#define BT_DW_CFA_def_cfa			0x0c
#define BT_DW_CFA_def_cfa_register		0x0d
#define BT_DW_CFA_def_cfa_offset		0x0e
#define BT_DW_CFA_def_cfa_expression		0x0f
#define BT_DW_CFA_expression			0x10
#define BT_DW_CFA_offset_extended_sf		0x11
#define BT_DW_CFA_def_cfa_sf			0x12
#define BT_DW_CFA_def_cfa_offset_sf		0x13
#define BT_DW_CFA_val_offset			0x14
#define BT_DW_CFA_val_offset_sf			0x15
#define BT_DW_CFA_val_expression		0x16
#define BT_DW_CFA_MIPS_advance_loc8		0x1d
#define BT_DW_CFA_GNU_window_save		0x2d
#define BT_DW_CFA_GNU_args_size			0x2e
#define BT_DW_CFA_lo_user			0x1c
#define BT_DW_CFA_hi_user			0x3f

/* DWARF exception handling extensions */
#define BT_DW_EH_PE_absptr			0x00
#define BT_DW_EH_PE_omit			0xff
#define BT_DW_EH_PE_leb128			0x01
#define BT_DW_EH_PE_data2			0x02
#define BT_DW_EH_PE_data4			0x03
#define BT_DW_EH_PE_data8			0x04
#define BT_DW_EH_PE_type_mask			0x07
#define BT_DW_EH_PE_signed			0x08

/* DWARF exception handling flags */
#define BT_DW_EH_PE_pcrel			0x10
#define BT_DW_EH_PE_textrel			0x20
#define BT_DW_EH_PE_datarel			0x30
#define BT_DW_EH_PE_funcrel			0x40
#define BT_DW_EH_PE_aligned			0x50
#define BT_DW_EH_PE_mod_mask			0x70
#define BT_DW_EH_PE_indirect			0x80

struct bt_bf;
struct bt_dwarf_die;
struct bt_dwarf;
struct bt_dwarf_call_site;
struct bt_dwarf_cu;
struct bt_frame;
struct bt_variable;
struct bt_variable_context;

enum bt_dwarf_debug_class {
	BT_DWARF_DEBUG_CLASS_32 = 0,
	BT_DWARF_DEBUG_CLASS_64
};

static inline bool
bt_dwarf_language_assembly(unsigned int lang)
{

	switch (lang) {
	case BT_DW_LANG_Mips_Assembler:
		return true;
	default:
		break;
	}

	return false;
}

/* Resolve to previous instruction. */
#define BT_DWARF_LVM_PREVIOUS	(1UL << 0)

/*
 * These are the states of the DWARF line state machine.
 */
enum bt_dwarf_lvm_state {
	BT_DWARF_LVM_STATE_START = 0,
	BT_DWARF_LVM_STATE_PHDR,
	BT_DWARF_LVM_STATE_RESOLVING,
	BT_DWARF_LVM_STATE_STOP,
	BT_DWARF_LVM_STATE_COMPLETED,
	BT_DWARF_LVM_STATE_FAIL
};

/*
 * The size class of the LVM.
 */
enum bt_dwarf_lvm_class {
	BT_DWARF_LVM_CLASS_32 = 0,
	BT_DWARF_LVM_CLASS_64
};

/*
 * File entry for DWARF line number programs.
 */
struct bt_dwarf_lvm_file {
	const char *path;
	const char *directory;
};

struct bt_dwarf_line {
	unsigned long long address;
	const char *directory;
	const char *path;
	unsigned long long line;

	/* Pointer to file block. */
	const unsigned char *block;
};

const char *bt_dwarf_lvm_file(struct bt_dwarf_line *, unsigned long long);

struct bt_dwarf_lvm_vm {
	enum bt_dwarf_lvm_class class;
	enum bt_dwarf_lvm_state state;

	struct {
		uint64_t unit_length;

		/* Version associated with block. */
		uint16_t version;

		/* Number of bytes from to the first program. */
		uint64_t header_length;

		/* Minimum instruction length, in case you couldn't tell. */
		uint8_t minimum_instruction_length;

		/* DWARF4: Maximum operationgs per instruction. */
		uint8_t maximum_operations_per_instruction;

		/* Default value for is_stmt. */
		uint8_t default_is_stmt;

		/* This affects the meaning of special opcodes. */
		int8_t line_base;

		/* This affects the meaning of special opcodes. */
		uint8_t line_range;

		/* The number assigned to the first special opcode. */
		uint8_t opcode_base;
	} phdr;

	const unsigned char *block;	/* Pointer to current file definition block. */
	const unsigned char *data;	/* Raw data for section. */
	const unsigned char *cursor;	/* Current cursor position. */
	const unsigned char *end;	/* Last byte of section. */
	const unsigned char *cu_end;	/* Last byte for compilation unit. */
	uintptr_t address;
	uint64_t op_index;		/* DWARF 4 */
	uint64_t file;
	uint64_t line;
	uint64_t column;
	bool is_stmt;
	bool basic_block;
	bool end_sequence;
	bool prologue_end;
	bool epilogue_begin;
	uint64_t isa;
	uint64_t discriminator;		/* DWARF 4 */

	struct {
		uint64_t *opcode_length;
		size_t capacity;
	} opcode;

	struct {
		const char **entry;
		size_t length;
		size_t capacity;
	} directory;

	struct {
		struct bt_dwarf_lvm_file *entry;
		size_t length;
		size_t capacity;
	} files;
};

#define BT_DWARF_LVM_LINE_SPLAY	128
#define BT_DWARF_LVM_LINE_SPLAY_MASK	\
	(BT_DWARF_LVM_LINE_SPLAY - 1)

struct bt_dwarf_lvm {
	unsigned long long base;
	struct bt_dwarf_lvm_vm vm;
	enum bt_dwarf_lvm_class class;

	struct {
		bool sorted;
		struct bt_dwarf_line *entry;
		size_t length;
		size_t capacity;
	} lines[BT_DWARF_LVM_LINE_SPLAY];
};
typedef struct bt_dwarf_lvm bt_dwarf_lvm_t;

bool bt_dwarf_lvm_bf_init(bt_dwarf_lvm_t *, struct bt_bf *);
bool bt_dwarf_lvm_resolve(bt_dwarf_lvm_t *, unsigned long,
    struct bt_dwarf_line *, unsigned long);

struct bt_dwarf_die_attr {
	unsigned long long value;
	unsigned long long length;
};

#define BT_DWARF_DIE_F_ARTIFICIAL	(1UL << 0)
#define BT_DWARF_DIE_F_CONSTANT		(1UL << 1)
#define BT_DWARF_DIE_F_LOCATION		(1UL << 2)
#define BT_DWARF_DIE_F_TYPE		(1UL << 3)
/* We have cached the DIE's address range. */
#define BT_DWARF_DIE_F_PC_CACHED	(1UL << 4)
/* We failed to compute the DIE's address range. */
#define BT_DWARF_DIE_F_PC_CACHE_FAIL	(1UL << 5)
/*
 * The DIE's address range is specified by AT_low_pc and AT_high_pc attributes.
 */
#define BT_DWARF_DIE_F_PC_LOW_HIGH	(1UL << 6)
/* The DIE's address ranges are specified by an AT_ranges attribute. */
#define BT_DWARF_DIE_F_PC_RANGES	(1UL << 7)
/* The DIE has an abstract origin */
#define BT_DWARF_DIE_F_ABSTRACT_ORIGIN	(1UL << 8)
/* The DIE references a specification DIE */
#define BT_DWARF_DIE_F_SPECIFICATION	(1UL << 9)

struct bt_dwarf_attr_pair {
	struct bt_dwarf_die_attr *die;
	struct bt_dwarf_attribute *attribute;
};

bool bt_dwarf_die_attr_get(struct bt_dwarf_die *, unsigned long long,
    struct bt_dwarf_die_attr **, struct bt_dwarf_attribute **);

static inline bool
bt_dwarf_attribute_get(struct bt_dwarf_die *die, unsigned long long a,
    struct bt_dwarf_attr_pair *p)
{

	return bt_dwarf_die_attr_get(die, a, &p->die, &p->attribute);
}

bool bt_dwarf_die_either_attr_get(struct bt_dwarf_die *, unsigned long long,
    unsigned long long, struct bt_dwarf_die_attr **,
    struct bt_dwarf_attribute **);

static inline bool
bt_dwarf_either_attribute_get(struct bt_dwarf_die *die, unsigned long long a,
    unsigned long long b, struct bt_dwarf_attr_pair *p)
{

	return bt_dwarf_die_either_attr_get(die, a, b, &p->die, &p->attribute);
}

struct bt_dwarf_die_specification {
	struct bt_dwarf_die *die;
	struct bt_dwarf *dwarf;
};

#define BT_DWARF_DIE_AO_NONE	((void *)-1)

/*
 * Debugging information entry (DIE) abstraction.
 */
struct bt_dwarf_die {
	/* Parser may cache attributes in flags. */
	unsigned long flags;
	struct {
		const char *name;
		unsigned long line;
		unsigned char accessibility;
		struct bt_dwarf_attr_pair location;
		struct bt_dwarf_attr_pair type;
		unsigned long long low_pc;
		unsigned long long high_pc;
		unsigned long long ao_offset;
	} cache;
	struct bt_dwarf_cu *cu;
	struct bt_dwarf_die *ao;
	struct bt_dwarf_die *parent;
	struct bt_dwarf_die_specification specification;
	unsigned long long offset;
	unsigned long long tag;
	struct bt_dwarf_abbrev *abbrev;
	STAILQ_HEAD(, bt_dwarf_die) children;
	STAILQ_ENTRY(bt_dwarf_die) linkage;

	/* Auxiliary linkage for various caches. */
	union {
		STAILQ_ENTRY(bt_dwarf_die) tlinkage;
		STAILQ_ENTRY(bt_dwarf_die) slinkage;
	} aux;

	SLIST_HEAD(, bt_dwarf_call_site) cs;
	struct bt_dwarf_die_attr attr[];
};

/*
 * Returns a DIE's abstract origin if it has one and NULL otherwise
 */
struct bt_dwarf_die *bt_dwarf_die_abstract_origin(struct bt_dwarf_die *);

/*
 * Returns the type for a DIE if available
 */
struct bt_type *bt_dwarf_die_type_get(struct bt_dwarf_die *);

/*
 * Returns a type from a name.
 */
struct bt_type *bt_dwarf_type_from_name(struct bt_dwarf *, const char *);

/*
 * Returns a DIE's name
 */
const char *bt_dwarf_die_name(struct bt_dwarf_die *);

static inline struct bt_dwarf_cu *
bt_dwarf_die_cu(struct bt_dwarf_die *die)
{

	return die->cu;
}

static inline unsigned long long
bt_dwarf_die_offset(const struct bt_dwarf_die *die)
{

	return die->offset;
}

enum bt_dwarf_variable_result {
	BT_DWARF_VARIABLE_OK,
	BT_DWARF_VARIABLE_IGNORE,
	BT_DWARF_VARIABLE_ERROR
};

/*
 * Create a new variable based on a DIE and add it to the relevant context.
 */
enum bt_dwarf_variable_result bt_dwarf_variable_die(struct bt_variable **,
    struct bt_dwarf_die *, struct bt_variable_context *, struct bt_dwarf *,
    unsigned long long);

/*
 * DIE tree has been processed.
 */
#define	BT_DWARF_CU_S_DIE		(1U << 0)
#define	BT_DWARF_CU_S_INVALID		(1U << 1)

/*
 * Compilation unit DIE abstraction.
 */
struct bt_dwarf_cu {
	/* Region with-in .debug_info section. */
	unsigned long long start;
	unsigned long long end;
	unsigned long long abbrev_offset;
	unsigned long long die_offset;
	unsigned long long low_pc;
	unsigned long long high_pc;
	unsigned int state;
	unsigned int addr_size;
	unsigned int ref;
	unsigned int version;
	enum bt_dwarf_debug_class class;
	const char *name;
	struct bt_dwarf *dwarf;
	STAILQ_HEAD(, bt_dwarf_die) die;
	STAILQ_HEAD(, bt_dwarf_die) types;
	STAILQ_HEAD(, bt_dwarf_die) subprograms;
	STAILQ_HEAD(, bt_dwarf_call_site) call_site;
	RB_ENTRY(bt_dwarf_cu) linkage;
};

static inline const char *
bt_dwarf_cu_name(const struct bt_dwarf_cu *cu)
{

	return cu->name;
}

static inline struct bt_dwarf *
bt_dwarf_cu_dwarf(struct bt_dwarf_cu *cu)
{

	return cu->dwarf;
}

static inline unsigned long long
bt_dwarf_cu_start(const struct bt_dwarf_cu *cu)
{

	return cu->start;
}

static inline void
bt_dwarf_cu_addref(struct bt_dwarf_cu *cu)
{

	cu->ref++;
	return;
}

static inline bool
bt_dwarf_cu_delref(struct bt_dwarf_cu *cu)
{

	if (cu == NULL)
		return false;

	return --cu->ref == 0;
}

/* Section's memory should be freed upon destruction */
#define BT_DWARF_SECTION_FREE		(1UL << 0)

struct bt_dwarf_section {
	unsigned char *start;		/* Start of section. */
	const unsigned char *cursor;	/* Current cursor position. */
	const unsigned char *end;	/* End of section. */
	const unsigned char *u_end;	/* End of unit. */
	unsigned long long offset;	/* Offset from begining of file */
	unsigned long flags;
};

enum bt_dwarf_debug_state {
	BT_DWARF_DEBUG_STATE_HEADER = 0,
	BT_DWARF_DEBUG_STATE_RESOLVE,
	BT_DWARF_DEBUG_STATE_STOP,
	BT_DWARF_DEBUG_STATE_COMPLETED,
	BT_DWARF_DEBUG_STATE_FAIL
};

/*
 * .debug_aranges entry.
 */
struct bt_dwarf_aranges {
	unsigned long long address;
	unsigned long long length;
	unsigned long long die_offset;
	struct bt_dwarf_cu *cu;
};

/*
 * Attribute entry.
 */
struct bt_dwarf_attribute {
	unsigned long long name;
	unsigned int form;
	unsigned int form_class;
};

/*
 * Attribute length.
 */
struct bt_dwarf_attributes {
	struct bt_dwarf_attribute *entry;
	unsigned int length;
	unsigned int capacity;
};

/*
 * .debug_abbrev key.
 */
struct bt_dwarf_abbrev_key {
	unsigned long long offset;
	unsigned long long code;
};

/*
 * .debug_abbrev entry.
 */
struct bt_dwarf_abbrev {
	struct bt_dwarf_abbrev_key key;
	unsigned long long tag;
	bool children;
	struct bt_dwarf_attributes attributes;
};

struct bt_dwarf {
	enum bt_dwarf_debug_class class;

	unsigned long long relocate;
	unsigned long long base_addr;

	/* Used for TLS */
	unsigned long long link_map;
	unsigned long long tls_base;

	struct bt_dwarf_section debug_aranges;
	struct bt_dwarf_section debug_info;
	struct bt_dwarf_section debug_loc;
	struct bt_dwarf_section debug_abbrev;
	struct bt_dwarf_section debug_str;
	struct bt_dwarf_section debug_types;
	struct bt_dwarf_section debug_ranges;
	struct bt_dwarf_section debug_frame;

	struct {
		unsigned int address_size;
		unsigned int version;
		enum bt_dwarf_debug_state state;
		enum bt_dwarf_debug_class class;
		struct bt_dwarf_section section;
		unsigned long long die_offset;
		struct bt_dwarf_aranges *entry;
		size_t length;
		size_t capacity;
	} aranges;

	struct {
		RB_HEAD(dwarf_cu, bt_dwarf_cu) cache;
		const unsigned char *cursor;
	} cu;

	struct {
		ck_hs_t cache;
		ck_hs_t ref_sig8;
		SLIST_HEAD(, bt_type) names;
	} type;

	struct {
		ck_hs_t cache;
	} abbrev;

	struct {
		/* Resolves DIE given file offset. */
		ck_hs_t cache;
	} die;

	struct bt_asset *asset;

	/* Optional alternate section file. */
	struct bt_dwarf *alternate;
};
typedef struct bt_dwarf bt_dwarf_t;

bool bt_dwarf_bf_init(bt_dwarf_t *, struct bt_bf *);
struct bt_dwarf_aranges *bt_dwarf_aranges_resolve_pc(bt_dwarf_t *,
    unsigned long long);
struct bt_dwarf_die *bt_dwarf_die_get(struct bt_dwarf *, unsigned long long);

/*
 * Initializes a DWARF section from the specified object section.
 */
bool bt_dwarf_bf_section(struct bt_dwarf_section *, struct bt_bf *, const char *); 

/*
 * Finds a DIE with the specified offset starting at the root of the tree
 * specified in the first argument. This operation is slow.
 */
struct bt_dwarf_die *bt_dwarf_die_find(struct bt_dwarf_die *,
    unsigned long long);

/*
 * Returns true iff die has both low_pc and high_pc attributes.  If the return
 * value is true then the output arguments will be populated with the low and
 * high addresses of the DIE, respectively.  The output arguments are both
 * optional and can be NULL.  If the return value is false then the output
 * contents of the output arguments are undefined.
 */
bool bt_dwarf_die_low_high(struct bt_dwarf_die *die, unsigned long long *,
    unsigned long long *);

bool bt_dwarf_die_add(struct bt_dwarf *, struct bt_dwarf_die *);
void bt_dwarf_destroy(struct bt_dwarf *);

static inline size_t
bt_dwarf_ptr_size(const struct bt_dwarf *d)
{

	switch (d->class) {
	case BT_DWARF_DEBUG_CLASS_32:
		return 4;
	case BT_DWARF_DEBUG_CLASS_64:
		return 8;
	default:
		break;
	}

	return sizeof(void *);
}

void bt_dwarf_asset_set(struct bt_dwarf *, struct bt_asset *);

struct bt_dwarf_cursor {
	const unsigned char *start;
	const unsigned char *cursor;
	const unsigned char *end;
	const unsigned char *u_end;
};

/*
 * Scratch buffer for parsing.
 */
union bt_dwarf_sb {
	uint8_t u8;
	uint16_t u16;
	uint32_t u32;
	uint64_t u64;
	int8_t s8;
	int16_t s16;
	int32_t s32;
	int64_t s64;
};

#ifndef BT_DWARF_EXPR_STACK_SIZE
#define BT_DWARF_EXPR_STACK_SIZE 256
#endif /* BT_DWARF_EXPR_STACK_SIZE */

#define BT_DWARF_EXPR_SIGN	(1U << 0)
#define BT_DWARF_EXPR_STACK	(1U << 1)
#define BT_DWARF_EXPR_REG	(1U << 2)
#define BT_DWARF_EXPR_ADDR	(1U << 3)
#define BT_DWARF_EXPR_VALUE	(1U << 4)
#define BT_DWARF_EXPR_PIECE	(1U << 5)
#define BT_DWARF_EXPR_REF	(1U << 6)
#define BT_DWARF_EXPR_ENTRY_VAL	(1U << 7)
#define BT_DWARF_EXPR_ANY	(UINT_MAX)

struct bt_dwarf_expr_value {
	unsigned int flags;
	unsigned long long value;
};

enum bt_dwarf_expr_result {
	BT_DWARF_EXPR_NONE = 1,
	BT_DWARF_EXPR_IMM,
	BT_DWARF_EXPR_MEM,
	BT_DWARF_EXPR_CMP
};

struct bt_dwarf_expr {
	enum bt_dwarf_expr_result rt;
	enum bt_dwarf_debug_class class;
	struct bt_dwarf_cursor cursor;
	struct bt_unwind_cursor *un;
	bt_register_mask_t reg_mask;
	unsigned int head;

	struct bt_dwarf_expr_value stack[BT_DWARF_EXPR_STACK_SIZE];
};

/*
 * .debug_call_site key.
 */
struct bt_dwarf_cs_key {
	unsigned long long ret_pc;
};

#define BT_DWARF_CS_F_RETURN_ADJUSTED	(1U << 0)
/* Callsite represents a tail call */
#define BT_DWARF_CS_F_TAIL_CALL		(1U << 1)
/*
 * Callsite DIE has an abstact_origin attribute pointing to the target function.
 */
#define BT_DWARF_CS_F_AO		(1U << 2)
/*
 * Callsite DIE has a target attribute which we must evaluate to determine the
 * target address.
 */
#define BT_DWARF_CS_F_TARGET_ATTR	(1U << 3)

struct bt_dwarf_call_site {
	struct bt_dwarf_cs_key key;
	struct bt_dwarf *dwarf;
	struct bt_dwarf_cu *cu;
	struct bt_dwarf_die *cs_die;
	union {
		unsigned long long callee_tag;
		struct bt_dwarf_attr_pair attr;
	} target;
	unsigned int flags;
	STAILQ_ENTRY(bt_dwarf_call_site) linkage;
	SLIST_ENTRY(bt_dwarf_call_site) fn_linkage;
};

CK_CC_CONTAINER(struct bt_dwarf_cs_key, struct bt_dwarf_call_site, key,
    bt_dwarf_cs_container)

#define BT_DWARF_RESOLVE_INLINE		(1UL << 0)
#define BT_DWARF_RESOLVE_VARIABLE	(1UL << 1)
#define BT_DWARF_RESOLVE_ARANGES	(1UL << 4)

/* Indicates that the current frame is for an inlined function */
#define BT_DWARF_FUNCTION_INLINE	(1UL << 2)

/*
 * Indicates that only variables with full location information should be used.
 */
#define BT_DWARF_FUNCTION_LOCATION	(1U << 5)

bool bt_dwarf_frame_die(bt_dwarf_t *, unsigned long long, unsigned long,
    struct bt_dwarf_cu **, struct bt_dwarf_die **, struct bt_process *);

bool bt_dwarf_resolve_frame(struct bt_frame *, bt_dwarf_t *, unsigned long);

/**
 * @brief Brute search for CU associated with specified PC.
 */
struct bt_dwarf_cu *bt_dwarf_cu_pc(struct bt_dwarf *,
    unsigned long long, struct bt_dwarf_die **);

/**
 * @brief Search for CU contained with-in the specified offset.
 */
struct bt_dwarf_cu *bt_dwarf_cu_offset(struct bt_dwarf *,
    unsigned long long, struct bt_dwarf_die **);

/*
 * First argument is CU name, second argument is user-provided.
 */
typedef bool bt_dwarf_cu_iterator_filter_t(const char *, void *);

/*
 * Don't delete references to CUs after loading them.
 */
#define BT_DWARF_CU_ITER_F_PERSIST	(1UL << 0)

struct bt_dwarf_cu_iterator {
	bt_dwarf_t *dwarf;
	const unsigned char *cursor;
	struct bt_dwarf_cu *last;
	bt_dwarf_cu_iterator_filter_t *filter;
	void *filter_closure;
	unsigned long flags;
};
typedef struct bt_dwarf_cu_iterator bt_dwarf_cu_iterator_t;

void bt_dwarf_cu_iterator_init(bt_dwarf_cu_iterator_t *, bt_dwarf_t *,
    bt_dwarf_cu_iterator_filter_t *, void *, unsigned long);
void bt_dwarf_cu_iterator_close(bt_dwarf_cu_iterator_t *);
struct bt_dwarf_cu *bt_dwarf_cu_iterator_next(bt_dwarf_cu_iterator_t *);

void bt_dwarf_die_print(unsigned int, struct bt_dwarf_die *);
void bt_dwarf_die_destroy(struct bt_dwarf *, struct bt_dwarf_die *);

struct bt_dwarf_cu *bt_dwarf_cu_get(bt_dwarf_t *, unsigned long long);

/*
 * Create a CU from the given DWARF object at the offset specified by the second
 * argument.  Also immediately load any data specified by flags in the third
 * argument.
 */
struct bt_dwarf_cu *bt_dwarf_cu_create(bt_dwarf_t *, unsigned long long,
    unsigned long);

/*
 * Load DIE information for the given CU.
 */
bool bt_dwarf_cu_load(struct bt_dwarf_cu *);

/*
 * Hoist type information from the specified CU into the specified process'
 * cache.
 */
void bt_dwarf_cu_types_hoist(struct bt_process *, struct bt_dwarf_cu *);

/*
 * Hoists subprogram information from the specified CU into the specified
 * process' cache.
 */
struct bt_process_cu_subprograms;
void bt_dwarf_cu_subprograms_hoist(struct bt_process_cu_subprograms *,
    struct bt_dwarf_cu *);

void bt_dwarf_cu_destroy(struct bt_dwarf_cu *, bt_dwarf_t *);
struct bt_dwarf_die *bt_dwarf_cu_resolve_pc(struct bt_dwarf_cu *,
    bt_dwarf_t *, unsigned long long);

/* Recursively evaluate children of the DIE */
#define BT_DWARF_DIE_READ_CHILD		(1UL << 0)

/*
 * Store any created DIEs in the containing DWARF object's records.
 * If this flag is not set, then the caller is responsible for disposing
 * of the DIE when they are done using it.
 */
#define BT_DWARF_DIE_READ_STORE		(1UL << 1)

struct bt_dwarf_die *bt_dwarf_die_read(struct bt_dwarf_section *,
    struct bt_dwarf_cu *, bt_dwarf_t *, unsigned long long,
    unsigned long long, enum bt_dwarf_debug_class, unsigned int,
    unsigned int, unsigned long);

struct bt_dwarf_abbrev *bt_dwarf_abbrev_get(bt_dwarf_t *,
    unsigned long long abbrev_offset, unsigned long long abbrev_code);
bool bt_dwarf_abbrev_init(bt_dwarf_t *);
bool bt_dwarf_die_init(bt_dwarf_t *);
bool bt_dwarf_cs_init(struct bt_process *);
void bt_dwarf_cs_hoist(struct bt_process *, struct bt_dwarf_cu *);
struct bt_dwarf_call_site *bt_dwarf_cs_create(bt_dwarf_t *,
    struct bt_dwarf_die *);
bool bt_dwarf_cs_entry_val(struct bt_process *, struct bt_dwarf_expr *,
    bt_error_t *);

/*
 * Find the longest unambiguous tail call chain between caller and callee
 * addresses.  Returns false if no tail call, ambiguous or unambiguous, exists.
 * If the return value is true, the caller is responsible for freeing the
 * resulting array. If there are unincluded ambiguous tail calls, a flag is
 * returned true along with the number of unambiguous tail calls before them.
 */
bool bt_dwarf_cs_tailcall_chain(struct bt_process *, unsigned long long,
    unsigned long long, struct bt_dwarf_call_site ***, unsigned int *, bool *,
    unsigned int *);

struct bt_dwarf_sig8 {
	/* Type signature must always be the first field. */
	unsigned long long type_signature;
	unsigned long long start;
	unsigned long long unit_length;
	unsigned long long debug_abbrev_offset;
	unsigned int address_size;
	unsigned long long type_offset;
	struct bt_dwarf_die *die;
};

bool bt_dwarf_sig8_init(struct bt_dwarf *);
struct bt_dwarf_sig8 *bt_dwarf_sig8_get(struct bt_dwarf *,
    unsigned long long);

bool bt_dwarf_ranges(unsigned long long, struct bt_dwarf *,
    unsigned long long, unsigned long long);

struct bt_dwarf_expr_context {
	enum bt_dwarf_debug_class class;
	unsigned long long relocate;
	unsigned long long tls_base;
	unsigned long long *link_map;
};

/*
 * The expression evaluates a TLS variable and therefore must be evaluated using
 * a thread context.
 */
#define BT_DWARF_EXPR_TLS		(1UL << 0)

/*
 * Evaluate a DWARF expression given the provided context.
 *
 * The expression stack will be pre-populated with values indicated by
 * prefill and n_prefill.
 *
 * If the flags argument is not NULL, it will have flags set to indicate
 * features of the expression being evaluated.  These flags will be set even if
 * the function returns false.
 */
bool bt_dwarf_expr(struct bt_dwarf_expr *expr,
    struct bt_dwarf_expr_context *context, struct bt_dwarf_cursor *cursor,
    struct bt_unwind_cursor *unwind_cursor, struct bt_dwarf_expr_value *prefill,
    size_t n_prefill, unsigned long *flags, bt_error_t *);

/*
 * Evaluate a DWARF expression using the provided DWARF object as context.
 * See the above function for details.
 */
bool bt_dwarf_expr_dwarf(struct bt_dwarf_expr *, struct bt_dwarf *,
    struct bt_dwarf_cursor *, struct bt_unwind_cursor *,
    struct bt_dwarf_expr_value *, size_t, unsigned long *,
    bt_error_t *);
bool bt_dwarf_expr_set(struct bt_dwarf_expr *, struct bt_dwarf_expr_value *,
    size_t, enum bt_dwarf_debug_class, struct bt_unwind_cursor *);
bool bt_dwarf_loc(struct bt_dwarf_expr *, struct bt_dwarf_cursor *, struct bt_dwarf *,
    unsigned long long, struct bt_unwind_cursor *,
    struct bt_dwarf_expr_value *, size_t, bt_error_t *);
bool bt_dwarf_expr_value_cflt(struct bt_dwarf_expr *, float complex *,
    unsigned int, bt_error_t *);
bool bt_dwarf_expr_value_cdbl(struct bt_dwarf_expr *, double complex *,
    unsigned int, bt_error_t *);
bool bt_dwarf_expr_value_cldbl(struct bt_dwarf_expr *, long double complex *,
    unsigned int, bt_error_t *);
bool bt_dwarf_expr_value_flt(struct bt_dwarf_expr *, float *,
    unsigned int, bt_error_t *);
bool bt_dwarf_expr_value_dbl(struct bt_dwarf_expr *, double *,
    unsigned int, bt_error_t *);
bool bt_dwarf_expr_value_ldbl(struct bt_dwarf_expr *, long double *,
    unsigned int, bt_error_t *);
bool bt_dwarf_expr_value_ull(struct bt_dwarf_expr *, unsigned long long *,
    unsigned int, unsigned int, unsigned int, bt_error_t *);
bool bt_dwarf_expr_value_ll(struct bt_dwarf_expr *, long long *,
    unsigned int, unsigned int, unsigned int, bt_error_t *);
bool bt_dwarf_expr_ull(struct bt_dwarf_expr *, unsigned long long *,
    unsigned long long, bt_error_t *);
bool bt_dwarf_expr_ull_type(struct bt_dwarf_expr *, unsigned long long *,
    unsigned long long *);
bool bt_dwarf_type_get(struct bt_type **, struct bt_dwarf *,
    struct bt_dwarf_attr_pair *, struct bt_dwarf_sig8 *);

struct bt_variable;

bool bt_dwarf_type_init(struct bt_dwarf *);

#define BT_DWARF_READ(X, C, L) do {		\
	memcpy((X), (C)->cursor, (L));		\
	(C)->cursor += (L);			\
} while (0)

#define BT_DWARF_PUN(X, C)		\
	BT_DWARF_READ(X, C, sizeof(*(X)))

/*
 * The 8th bit indicates whether we should continue streaming
 * into the result accumulator.
 */
static inline uint64_t
bt_dwarf_uleb128(const unsigned char **r, const unsigned char *end)
{
	uint64_t result = 0;
	unsigned int shift = 0;
	const unsigned char *cursor = *r;

	while (cursor < end) {
		unsigned char v = *cursor++;

		result |= (uint64_t)(v & 0x7F) << shift;
		if ((v & 0x80) == 0)
			break;

		shift += 7;
	}

	*r = cursor;
	return result;
}

/*
 * Same semantics as above except that if the last bit has a sign bit
 * set then the over-all value is sign extended.
 */
static inline int64_t
bt_dwarf_leb128(const unsigned char **r, const unsigned char *end)
{
	int64_t sr;
	uint64_t result = 0;
	unsigned int shift = 0;
	const unsigned char *cursor = *r;
	bool sign = false;

	while (cursor < end) {
		unsigned char v = *cursor++;

		result |= (uint64_t)(v & 0x7F) << shift;
		shift += 7;
		sign = (v & 0x40);
		if ((v & 0x80) == 0)
			break;
	}

	sr = (int64_t)result;
	if ((shift < 64) & sign)
		sr |= -(1 << shift);

	*r = cursor;
	return sr;
}

static inline bool
bt_dwarf_attribute_ref_cu(struct bt_dwarf_attribute *a)
{
	return (a->form_class & BT_DW_FORM_CLS_ref_cu);
}

static inline bool
bt_dwarf_attribute_ref(struct bt_dwarf_attribute *a)
{
	return (a->form_class & BT_DW_FORM_CLS_ref);
}

static inline bool
bt_dwarf_attribute_data(struct bt_dwarf_attribute *a)
{
	return (a->form_class & BT_DW_FORM_CLS_constant);
}


static inline bool
bt_dwarf_attribute_constant(struct bt_dwarf_attribute *a)
{
	return (a->form_class &
	      (BT_DW_FORM_CLS_constant | BT_DW_FORM_CLS_constant_us));
}

static inline bool
bt_dwarf_attribute_sect_offset(struct bt_dwarf_attribute *a)
{
	return (a->form_class & BT_DW_FORM_CLS_sec_offset);
}

static inline bool
bt_dwarf_attribute_block_only(struct bt_dwarf_attribute *a)
{
	return (a->form_class & BT_DW_FORM_CLS_block);
}

static inline bool
bt_dwarf_attribute_block(struct bt_dwarf_attribute *a)
{
	return (a->form_class &
	      (BT_DW_FORM_CLS_block | BT_DW_FORM_CLS_exprloc));
}

static inline bool
bt_dwarf_attribute_address(struct bt_dwarf_attribute *a)
{
	return (a->form_class & BT_DW_FORM_CLS_addr);
}

static inline bool
bt_dwarf_attribute_flag(struct bt_dwarf_attribute *a)
{
	return (a->form_class & BT_DW_FORM_CLS_flag);
}

/*
 * Write the flag value of the attribute to the last argument.
 * The contents of the last argument are only valid if the return value is true.
 * Returns false if the attribute is not a flag attribute.
 */
bool bt_dwarf_attribute_flag_value(struct bt_dwarf_attr_pair *, bool *);

/*
 * Returns true if the specified tag attribute describes a type DIE.
 */
bool bt_dwarf_tag_is_type(unsigned long long);

/*
 * Enable DWARF-based unwinding support.
 * This includes support for .eh_frame.
 */
bool bt_dwarf_unwind_init(void);

#endif /* _BT_DWARF_H */
