#ifndef BT_TARGET_H
#define BT_TARGET_H

#include <stdbool.h>
#include <sys/types.h>

#include "bt_unwind.h"
#include "error.h"

struct bt_bf;
struct bt_dwarf;
struct bt_process;
struct bt_target_adapter;
struct bt_td_thread;
struct bt_thread;

enum bt_target_type {
	BT_TARGET_PROCESS = 0,
	BT_TARGET_CORE,
	BT_TARGET_KVM
};

/*
 * A target provides an interface for interacting with a source of process
 * information.  This can include live processes, core files, network services
 * that provide information about past processes, etc.
 * Library users should only interact with the bt_process interface, which
 * provides a normalized view of process information.  The bt_target interface
 * is used behind the scenes to collect process data and control process status
 * and normalize this behavior into a form suitable for bt_process.
 */

struct bt_target {
	enum bt_target_type type;
	const struct bt_target_adapter *adapter;
	void *md;
};
typedef struct bt_target bt_target_t;

static inline enum bt_target_type
bt_target_type(const struct bt_target *target)
{

	return target->type;
}

struct bt_target_options {

};

#define BT_TARGET_OPTIONS_INITIALIZER { }

/*
 * Initialize the image subsystem.  This function must be called before any
 * other image-related functionality is available.
 */
bool bt_target_init(void);

/*
 * Initialize a target from a process ID.
 * Returns true on success and false otherwise.
 * An error code of BT_ERROR_NA indicates that the process with the given
 * ID is not supported.
 * An error code of BT_ERROR_FAILURE indicates that an error occurred when
 * trying to create the target.
 */
bool bt_target_process(bt_target_t *, pid_t, struct bt_target_options *,
    bt_error_t *);

/*
 * Initialize a target from a core file and executable with the given paths.
 * Returns true on success and false otherwise.
 * An error code of BT_ERROR_NA indicates that the core file or executable type
 * is not supported.
 * An error code of BT_ERROR_FAILURE indicates that an error occurred when
 * trying to create the target.
 */
bool bt_target_core(bt_target_t *, const char *, const char *,
    struct bt_target_options *, bt_error_t *);

/*
 * Deinitialize a target.
 */
void bt_target_deinit(bt_target_t *);

#endif /* BT_TARGET_H */
